# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Sphinx configuration."""

from invenio_workflows_tugraz import __version__

# Do not warn on external images.
suppress_warnings = ["image.nonlocal_uri"]

# General information about the project.
project = "invenio-workflows-tugraz"
copyright = "2022, Graz University of Technology"
author = "Graz University of Technology"

nitpick_ignore = [
    ("py:class", "invenio_campusonline.types.CampusOnlineConfigs"),
    ("py:class", "invenio_pure.types.PureConfigs"),
]

# -- General configuration ---------------------------------------------------

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.coverage",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
]

templates_path = ["_templates"]
exclude_patterns = []

# The master toctree document.
master_doc = "index"

# The full version, including alpha/beta/rc tags.
release = __version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# -- Options for HTML output -------------------------------------------------

html_theme = "alabaster"

html_theme_options = {
    "description": "This package serves as a place for the workflows of the repository of the TU Graz.",
    "github_user": "tu-graz-library",
    "github_repo": "invenio-workflows-tugraz",
    "github_button": False,
    "github_banner": True,
    "show_powered_by": False,
    "extra_nav_links": {
        "invenio-workflows-tugraz@GitHub": "https://github.com/tu-graz-library/invenio-workflows-tugraz",
        "invenio-workflows-tugraz@PyPI": "https://pypi.python.org/pypi/invenio-workflows-tugraz/",
    },
}

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    "**": [
        "about.html",
        "navigation.html",
        "relations.html",
        "searchbox.html",
        "donate.html",
    ]
}



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Theses Workflows."""

from .theses import (
    import_func,
    theses_create_aggregator,
    theses_filter_for_locked_records,
    theses_filter_for_open_records,
)

__all__ = (
    "theses_create_aggregator",
    "theses_filter_for_locked_records",
    "theses_filter_for_open_records",
    "import_func",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Theses Workflows."""


from invenio_records_marc21.services.record.types import Marc21Category


class CampusOnlineId(Marc21Category):
    """Campus online ID."""

    category: str = "995.subfields.d.keyword"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Open Access Workflow."""
from typing import Callable

from invenio_config_tugraz import get_identity_from_user_by_email
from invenio_pure import PureConfigs, PureRecord
from invenio_records_marc21 import (
    DuplicateRecordError,
    Marc21Metadata,
    check_about_duplicate,
    create_record,
    current_records_marc21,
)

from .convert import Pure2Marc21
from .types import PureId
from .utils import (
    access_type,
    extract_file_url,
    extract_pure_id,
    license_type,
    workflow,
)


def pure_import_func(
    pure_record: PureRecord,
    configs: PureConfigs,
    download_file: Callable,
) -> None:
    """Import record from pure into the repository."""
    pure_id = extract_pure_id(pure_record)
    file_urls = extract_file_url(pure_record)

    file_paths = []
    for i, file_url in enumerate(file_urls):
        file_path = download_file(
            f"{pure_id}-{i}", file_url, configs.pure_username, configs.pure_password
        )
        file_paths.append(file_path)

    marc21_record = Marc21Metadata()
    convert = Pure2Marc21()
    convert.convert(pure_record, marc21_record)

    identity = get_identity_from_user_by_email(email=configs.user_email)
    service = current_records_marc21.records_service
    data = marc21_record.json
    data["access"] = {"record": "public", "files": "public"}

    return create_record(service, data, file_paths, identity, do_publish=True)


def pure_sieve_func(pure_record: PureRecord) -> bool:
    """Check if the record fullfills the import criteria."""
    try:
        pure_id = extract_pure_id(pure_record)
        check_about_duplicate(PureId(pure_id))
        duplicate_sieve = True
    except DuplicateRecordError:
        return False

    file_sieve = False
    for electronic_version in pure_record["electronicVersions"]:
        if (
            "file" in electronic_version
            and access_type(electronic_version) in ["Open", "Offen"]
            and license_type(electronic_version).startswith("CC BY")
            and workflow(pure_record["workflow"]) in ["Valid"]
        ):
            file_sieve = True

    return duplicate_sieve and file_sieve



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Convert from CampusOnline to Marc21."""

from xml.etree.ElementTree import Element

from invenio_records_marc21.services.record.metadata import Marc21Metadata, QName


def construct_name(name):
    """Construct name."""
    return f"{name['ln']}, {name['fn']}"


def language_decode(lang):
    """Language decode."""
    languages = {
        "DE": "deu",
        "EN": "eng",
    }
    return languages.get(lang, "PLATZHALTER")


class Visitor:
    """Visitor base class."""

    def process(self, node: Element, record: Marc21Metadata):
        """Execute the corresponding method to the tag name."""

        def func_not_found(*args, **kwargs):
            localname = QName(node).localname
            namespace = QName(node).namespace
            raise ValueError(f"NO visitor node: '{localname}' ns: '{namespace}'")

        tag_name = QName(node).localname
        visit_func = getattr(self, f"visit_{tag_name}", func_not_found)
        result = visit_func(node, record)
        return result

    def visit_attr(self, node, record: Marc21Metadata):
        """Base function for attr."""
        key = node.attrib["key"]

        def func_not_found(*args, **kwargs):
            raise ValueError(f"NO visitor node: '{key}'")

        visit_func = getattr(self, f"visit_{key}", func_not_found)
        result = visit_func(node, record)
        return result

    def visit(self, node, record: Marc21Metadata):
        """Entry point for visitor."""
        for child in node:
            self.process(child, record)


class CampusOnlineToMarc21(Visitor):
    """Convertor from CampusOnline to Marc21."""

    def __init__(self, record: Marc21Metadata):
        """Constructor."""
        super().__init__()
        self.author_name = "N/A"
        self.state = ""
        self.metaclass_name = ""
        self.data_elements = ""

        record.emplace_controlfield("007", "cr#|||||||||||")
        record.emplace_datafield(
            "040...", subfs={"a": "AT-UBTUG", "b": "ger", "d": "AT-UBTUG", "e": "rda"}
        )
        record.emplace_datafield("044...", subfs={"c": "XA-AT"})
        record.emplace_datafield("264..1.", subfs={"a": "Graz", "c": "JAHR"})
        record.emplace_datafield(
            "300...", subfs={"a": "1 Online-Ressource ( Seiten )", "b": "ill"}
        )
        record.emplace_datafield("336...", subfs={"b": "txt"})
        record.emplace_datafield("337...", subfs={"b": "c"})
        record.emplace_datafield("338...", subfs={"b": "cr"})
        record.emplace_datafield("347...", subfs={"a": "Textdatei", "b": "PDF"})
        record.emplace_datafield(
            "506.0..", subfs={"f": "Unrestricted online access", "2": "star"}
        )
        record.emplace_datafield("546...", subfs={"a": "Zusammenfassung in"})
        record.emplace_datafield(
            "655...",
            subfs={
                "a": "Hochschulschrift",
                "0": "(DE-588)4113937-9",
                "d": "gnd-content",
            },
        )

    def visit(self, node, record: Marc21Metadata):
        """Override visit."""
        super().visit(node, record)
        # TODO: find out what in 008 should be
        record.emplace_controlfield("008", self.data_elements)

    def visit_ID(self, node, record: Marc21Metadata):
        """Visit ID."""
        record.emplace_datafield(
            "995...", subfs={"i": "TUGRAZonline", "d": node.text, "9": "local"}
        )

    def visit_PAG(self, node, record: Marc21Metadata):
        """Visit PAG."""

    def visit_CO(self, node: Element, record: Marc21Metadata):
        """Visit CO."""

    def visit_CHD(self, node: Element, record: Marc21Metadata):
        """Visit CHD."""

    def visit_EJAHR(self, node: Element, record: Marc21Metadata):
        """Visit EJAHR."""

    def visit_ARCHD(self, node: Element, record: Marc21Metadata):
        """Visit ARCHD."""

    def visit_PUBD(self, node: Element, record: Marc21Metadata):
        """Visit PUBD."""

    def visit_PUBLIC(self, node: Element, record: Marc21Metadata):
        """Visit PUBLIC."""

    def visit_STATUS(self, node: Element, record: Marc21Metadata):
        """Visit status."""

    def visit_STATUSD(self, node: Element, record: Marc21Metadata):
        """Visit Status date."""

    def visit_ORG(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_ORGP(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        orgp = node.text.split("&gt;")

        faculty = orgp[1] if len(orgp) > 1 else ""
        institute = orgp[2] if len(orgp) > 2 else ""

        record.emplace_datafield(
            "971.5..",
            subfs={"a": "Technische Universität Graz", "b": faculty, "c": institute},
        )

    def visit_TYPKB(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        value = "HS-DISS" if node.text == "DISS" else "HS-MASTER"
        record.emplace_datafield("970.2..", subfs={"d": value})

    def visit_TYP(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        if self.state == "metaobj":
            self.typ = node.text
            return

        record.emplace_datafield(
            "502...",
            subfs={"b": node.text, "c": "Technische Universität Graz", "d": "JAHR"},
        )

    def visit_ZUGKB(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_ZUG(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_SPSTAT(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_SPVON(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        text = node.text
        self.spvon = text.split(" ")[0] if text else ""

    def visit_SPBIS(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        text = node.text

        if text:
            record.emplace_datafield(
                "971.7..",
                subfs={"a": "gesperrt", "b": self.spvon, "c": text.split(" ")[0]},
            )

    def visit_SPBGR(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_OLANG(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        self.object_language = node.text
        record.emplace_datafield("041...a", value=language_decode(node.text))

    def visit_TLANGS(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_VOLLTEXT(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_metaclass(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        self.visit(node, record)

    def visit_name(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        self.metaclass_name = node.text

    def visit_metaobj(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        if self.metaclass_name not in ["AUTHOR", "TEXT", "SUPERVISOR"]:
            return

        self.state = "metaobj"
        self.name = {"fn": "", "ln": ""}

        self.visit(node, record)

        if self.metaclass_name == "AUTHOR":
            self.author_name = construct_name(self.name)
            record.emplace_datafield(
                "100.1..", subfs={"a": self.author_name, "4": "aut"}
            )

        if self.metaclass_name == "SUPERVISOR":
            types = {
                "BTTUG": "0",
                "MBTUG": "2",
                "1BUTUG": "1",
            }
            ind1 = types.get(self.typ, "")
            record.emplace_datafield(f"971.{ind1}.0.a", value=construct_name(self.name))

        self.state = ""

    def visit_FN(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        self.name["fn"] = node.text

    def visit_LN(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        self.name["ln"] = node.text

    def visit_FNLN(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_AKK(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_MNR(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_STRI(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_STKZ(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_ORIG(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_INTERN(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_TIT(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        if self.state == "metaobj" and self.language == self.object_language:
            record.emplace_datafield(
                "245.1.0.", subfs={"a": node.text, "c": self.author_name}
            )

        if self.state == "metaobj" and self.language != self.object_language:
            record.emplace_datafield(
                "246.1..", subfs={"i": "TUGRAZonline", "a": node.text}
            )

    def visit_ABS(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_KEYW(self, node: Element, record: Marc21Metadata):
        """Visit ."""

    def visit_LANG(self, node: Element, record: Marc21Metadata):
        """Visit ."""
        self.language = node.text



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""This package serves as a place for the workflows of the repository of the TU Graz."""

from . import config


class InvenioWorkflowsTugraz:
    """invenio-workflows-tugraz extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        app.extensions["invenio-workflows-tugraz"] = self

    def init_config(self, app):
        """Initialize configuration."""
        app.config.setdefault("ALMA_REPOSITORY_RECORDS_UPDATE_AGGREGATORS", [])
        app.config.setdefault("ALMA_ALMA_RECORDS_CREATE_AGGREGATORS", [])
        app.config.setdefault("CAMPUSONLINE_THESES_FILTERS", [])

        for k in dir(config):
            if k == "WORKFLOW_ALMA_REPOSITORY_RECORDS_UPDATE_AGGREGATORS":
                app.config["ALMA_REPOSITORY_RECORDS_UPDATE_AGGREGATORS"] += getattr(
                    config, k
                )

            elif k == "WORKFLOW_ALMA_ALMA_RECORDS_CREATE_AGGREGATORS":
                app.config["ALMA_ALMA_RECORDS_CREATE_AGGREGATORS"] += getattr(config, k)

            elif k == "WORKFLOW_CAMPUSONLINE_THESES_FILTERS":
                app.config["CAMPUSONLINE_THESES_FILTERS"] += getattr(config, k)

            elif k == "WORKFLOW_CAMPUSONLINE_IMPORT_FUNC":
                app.config["CAMPUSONLINE_IMPORT_FUNC"] = getattr(config, k)

            elif k.startswith("WORKFLOWS_TUGRAZ_"):
                app.config.setdefault(k, getattr(config, k))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Open Access Workflow."""

from json import load
from os.path import dirname, join

from invenio_records_marc21 import Marc21Metadata


class Converter:
    """Converter base class to convert one format into another."""

    def __init__(self):
        """Default Constructor of the class."""
        # Cache iso639-3 language codes to dict
        self.languages = self.initialize_languages()

    def initialize_languages(self) -> dict:
        """Initialize language cache dictionary."""
        path = join(dirname(__file__), "../data", "iso6393.json")

        with open(path) as fp:
            languages = filter(lambda obj: "iso6393" in obj, load(fp))

        return {language["name"]: language["iso6393"] for language in languages}

    def convert(self, value: object, record: Marc21Metadata):
        """Convert record from Pure JSON format to MARC21XML."""
        for attribute, value in value.items():
            self.convert_attribute(attribute, value, record)

    def convert_attribute(self, attribute: str, value: object, record: Marc21Metadata):
        """Traverse first level elements of dictionary and extract necessary attributes."""
        convert_function = getattr(self, f"convert_{attribute}", lambda *args: None)
        convert_function(value, record)


class Pure2Marc21(Converter):
    """Pure to marc21 converter class."""

    def convert_abstract(self, value: dict, record: Marc21Metadata):
        """Add the abstract to the Marc21Metadata."""
        for abstract in value["text"]:
            record.emplace_field("520...", value=abstract["value"])

    def convert_additionalLinks(self, value: list, record: Marc21Metadata):
        """Add the additionalLinks attribute to the Marc21Metadata."""
        for link in value:
            if "url" in link:
                record.emplace_field("856.4.1.u", value=link["url"])

    def convert_bibliographicalNote(self, value: dict, record: Marc21Metadata):
        """Add the bibliographicalNote attribute to the Marc21Metadata."""
        for text in value["text"]:
            record.emplace_field("500...", value=text["value"])

    def convert_edition(self, value: str, record: Marc21Metadata):
        """Add the edition attribute to the Marc21Metadata."""
        record.emplace_field("250...", value=value)

    def convert_electronicIsbns(self, value: list, record: Marc21Metadata):
        """Add the electronicIsbns attribute to the Marc21Metadata."""
        record.emplace_field("020...", value=str(value[0]).strip())

    def convert_event(self, value: dict, record: Marc21Metadata):
        """Add the event attribute to the Marc21Metadata."""
        for event_name in value["name"]["text"]:
            record.emplace_field("711.2..", value=event_name["value"])

    def convert_isbns(self, value: list, record: Marc21Metadata):
        """Add the isbns attribute to the Marc21Metadata."""
        for isbn in value:
            record.emplace_field("020...", value=isbn)

    def convert_journalAssociation(self, value: dict, record: Marc21Metadata):
        """Add the journalAssociation attribute to the Marc21Metadata."""
        value = value["title"]["value"]
        record.emplace_field("773.0.8.t", value=value)

    def convert_journalNumber(self, value: str, record: Marc21Metadata):
        """Add the journalNumber attribute to the Marc21Metadata."""
        record.emplace_field("773.0.8.g", value=value)

    def convert_keywordGroups(self, value: list, record: Marc21Metadata):
        """Add the keywordGroups attribute to the Marc21Metadata."""
        for item in value:
            keyword_group = KeywordGroup()
            keyword_group.convert(item, record)

    def convert_language(self, value: dict, record: Marc21Metadata):
        """Add the language attribute to the Marc21Metadata."""
        for locale in value["term"]["text"]:
            language = locale["value"]
            language_iso6393 = self.languages[language]
            record.emplace_field("041...", value=language_iso6393)

    def convert_managingOrganisationalUnit(self, value: dict, record: Marc21Metadata):
        """Add the managingOrganisationalUnit attribute to the Marc21Metadata."""
        for locale in value["name"]["text"]:
            record.emplace_unique_field("100.1..u", value=locale["value"])
            record.emplace_unique_field("700.1..u", value=locale["value"])

    def convert_numberOfPages(self, value: int, record: Marc21Metadata):
        """Add the numberOfPages attribute to the Marc21Metadata."""
        record.emplace_field("300...", value=str(value))

    def convert_organisationalUnits(self, value: list, record: Marc21Metadata):
        """Add the organisationalUnits attribute to the Marc21Metadata."""
        for o_unit in value:
            for locale in o_unit["name"]["text"]:
                record.emplace_unique_field("100.1..u", value=locale["value"])
                record.emplace_unique_field("700.1..u", value=locale["value"])

    def convert_pages(self, value: str, record: Marc21Metadata):
        """Add the pages attriute to the Marc21Metadata."""
        pages = value
        record.emplace_field("300...", value=pages)

    def convert_patentNumber(self, value: str, record: Marc21Metadata):
        """Add the patentNumber attribute to the Marc21Metadata."""
        record.emplace_field("013...", value=value)

    def convert_peerReview(self, value: bool, record: Marc21Metadata):
        """Add the peerReview attribute to the Marc21Metadata."""
        if value:
            status = "Refereed/Peer-reviewed"
            record.emplace_field("500...", value=status)

    def convert_placeOfPublication(self, value: str, record: Marc21Metadata):
        """Add the placeOfPublication attribute to the Marc21Metadata."""
        record.emplace_field("264...", value=value)

    def convert_publicationSeries(self, value: list, record: Marc21Metadata):
        """Add the publicationSeries attribute to the Marc21Metadata."""
        for series in value:
            record.emplace_field("490.0..", value=series["name"])

    def convert_publicationStatuses(self, value: list, record: Marc21Metadata):
        """Add the publicationStatuses attribute to the Marc21Metadata."""
        for entry in value:
            publication_status = PublicationStatus()
            publication_status.convert(entry, record)

    def convert_publisher(self, value: dict, record: Marc21Metadata):
        """Add the publisher attribute to the Marc21Metadata."""
        for text in value["name"]["text"]:
            record.emplace_field("264...b", value=text["value"])

    def convert_relatedProjects(self, value: list, record: Marc21Metadata):
        """Add the relatedProjects attribute to the Marc21Metadata."""
        for entry in value:
            for locale in entry["name"]["text"]:
                record.emplace_field("536...", value=locale["value"])

    def convert_subTitle(self, value: dict, record: Marc21Metadata):
        """Add the subTitle attribute to the Marc21Metadata."""
        record.emplace_field("245.1.0.b", value=value["value"])

    def convert_title(self, value: dict, record: Marc21Metadata):
        """Add the title attribute to the Marc21Metadata."""
        record.emplace_field("245.1.0.", value=value["value"])

    def convert_volume(self, value: str, record: Marc21Metadata):
        """Add the volume attribute to the Marc21Metadata."""
        record.emplace_field("490.0..", value=value)
        record.emplace_field("773.0.8.g", value=value)


class KeywordGroup(Converter):
    """Class to convert the keywordgroup attribute."""

    def convert_keywordContainers(self, value: list, record: Marc21Metadata):
        """Add keywords."""
        for item in value:
            self.convert(item, record)

    def convert_freeKeywords(self, value: list, record: Marc21Metadata):
        """Add free keywords."""
        for free_keyword in value:
            for word in free_keyword["freeKeywords"]:
                record.emplace_field("650..4.g", value=word)

    def convert_structuredKeyword(self, value: dict, record: Marc21Metadata):
        """Add free keywords."""
        for word in value["term"]["text"]:
            record.emplace_field("650..4.", value=word["value"])


class PublicationStatus(Converter):
    """Class to convert publication status."""

    def convert_publicationDate(self, value: dict, record: Marc21Metadata):
        """Add the publication date to the Marc21Metadata."""
        record.emplace_field("264...c", value=value["year"])

    def convert_publicationStatus(self, value: dict, record: Marc21Metadata):
        """Add the publication status to the Marc21Metadata."""
        for text in value["term"]["text"]:
            record.emplace_field("250...", value=text["value"])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""This is the configuration file."""

from .openaccess import pure_import_func, pure_sieve_func
from .theses import (
    import_func,
    theses_create_aggregator,
    theses_filter_for_locked_records,
    theses_filter_for_open_records,
)

WORKFLOW_ALMA_REPOSITORY_RECORDS_UPDATE_AGGREGATORS = []
""""""

WORKFLOW_ALMA_ALMA_RECORDS_CREATE_AGGREGATORS = [
    theses_create_aggregator,
]
""""""

WORKFLOW_CAMPUSONLINE_THESES_FILTERS = [
    theses_filter_for_locked_records(),
    theses_filter_for_open_records(),
]
""""""

WORKFLOW_CAMPUSONLINE_IMPORT_FUNC = import_func


WORKFLOW_PURE_IMPORT_FUNC = pure_import_func
"""See corresponding varaible in invenio-pure."""

WORKFLOW_PURE_SIEVE_FUNC = pure_sieve_func



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Theses Workflows."""

from typing import Callable
from xml.etree.ElementTree import Element

from invenio_campusonline.types import (
    CampusOnlineConfigs,
    CampusOnlineID,
    ThesesFilter,
    ThesesState,
)
from invenio_config_tugraz import get_identity_from_user_by_email
from invenio_records_marc21 import (
    DuplicateRecordError,
    Marc21Metadata,
    check_about_duplicate,
    create_record,
    current_records_marc21,
)
from invenio_search import RecordsSearch
from invenio_search.engine import dsl

from .convert import CampusOnlineToMarc21
from .types import CampusOnlineId


@check_about_duplicate.register
def _(value: CampusOnlineId):
    """Check about double campus online id."""
    check_about_duplicate(str(value), value.category)


def cms_id(record):
    """CMS id."""
    return record["_source"]["metadata"]["fields"]["995"][0]["subfields"]["d"][0]


def marc_id(record):
    """Marc id."""
    return record["_source"]["id"]


def theses_filter_for_locked_records():
    """This function returns a tuple.

    FILTER: xml filter to get locked records
    STATE: [open, locked]
    return ThesesFilter
    """
    filter_ = [
        """<bas:thesesType>ALL</bas:thesesType>""",
        """<bas:state name="LOCKED" negate="false"><bas:from>2022-01-01T00:01:00+00:00</bas:from></bas:state>""",
    ]
    state = ThesesState.LOCKED
    return ThesesFilter(filter_, state)


def theses_filter_for_open_records():
    """This function returns a list of tuples.

    FILTER: xml filter to get open records
    STATE: [open, locked]
    return ThesesFilter
    """
    filter_ = [
        """<bas:thesesType>ALL</bas:thesesType>""",
        """<bas:state name="IFG" negate="false"><bas:from>2022-01-01T00:01:00+00:00</bas:from></bas:state>""",
    ]
    state = ThesesState.OPEN
    return ThesesFilter(filter_, state)


def theses_create_aggregator():
    """This function returns a list of marc21 ids."""
    print("---------- theses_create_aggregator -----------------")
    search = RecordsSearch(index="marc21records-drafts")
    query = {
        "must_not": [
            {
                "exists": {
                    "field": "metadata.fields.001",
                },
            }
        ],
        "must": [
            {
                "exists": {
                    "field": "metadata.fields.995",
                },
            }
        ],
    }

    search.query = dsl.Q("bool", **query)
    search = search.params(size=50000)
    result = search.execute()
    hits = result["hits"]["hits"]
    return [(marc_id(record), cms_id(record)) for record in hits]


def theses_update_aggregator():
    """This function returns a list of tuple(marc21, cms_id)."""
    search = RecordsSearch(index="marc21records-draft")
    query = {
        "must_not": [
            {
                "exists": {
                    "field": "metadata.fields.001",
                },
            }
        ]
    }
    search.query = dsl.Q("bool", **query)
    result = search.execute()
    hits = result["hits"]["hits"]
    return [(marc_id(record), cms_id(record)) for record in hits]


def exists_fulltext(thesis: Element) -> bool:
    """Check against fulltext existens."""
    ns = "http://www.campusonline.at/thesisservice/basetypes"
    xpath = f".//{{{ns}}}attr[@key='VOLLTEXT']"
    ele = thesis.find(xpath)

    if ele is None:
        return False
    elif ele.text == "N":
        return False
    else:
        return True


def import_func(
    cms_id: CampusOnlineID,
    configs: CampusOnlineConfigs,
    get_metadata: Callable,
    download_file: Callable,
):
    """This is the function to import the record into the repository."""
    try:
        check_about_duplicate(CampusOnlineId(cms_id))
    except DuplicateRecordError:
        return

    thesis = get_metadata(configs.endpoint, configs.token, cms_id)

    if not exists_fulltext(thesis):
        raise RuntimeError("record has no associated file")

    file_path = download_file(configs.endpoint, configs.token, cms_id)

    marc21_record = Marc21Metadata()
    convert = CampusOnlineToMarc21(marc21_record)

    convert.visit(thesis, marc21_record)

    identity = get_identity_from_user_by_email(email=configs.user_email)
    service = current_records_marc21.records_service
    data = marc21_record.json
    data["access"] = {
        "record": "restricted",
        "files": "restricted",
    }

    return create_record(service, data, [file_path], identity, do_publish=False)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Open Access Workflow."""

from .workflow import pure_import_func, pure_sieve_func

__all__ = ("pure_import_func", "pure_sieve_func")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Openaccess Workflow utils."""

from invenio_pure import URL, PureID, PureRecord, PureRuntimeError
from invenio_records_marc21 import check_about_duplicate

from .types import PureId


@check_about_duplicate.register
def _(value: PureId):
    """Check about double pure id."""
    check_about_duplicate(str(value), value.category)


def access_type(electronic_version: dict) -> str:
    """Get Access type."""
    try:
        return electronic_version["accessType"]["term"]["text"][0]["value"]
    except (KeyError, TypeError):
        return False


def license_type(electronic_version: dict) -> str:
    """Get license type."""
    try:
        return electronic_version["licenseType"]["term"]["text"][0]["value"]
    except (KeyError, TypeError):
        return False


def workflow(electronic_version: dict) -> str:
    """Get workflow status."""
    try:
        return electronic_version["workflow"]["value"]["text"][0]["value"]
    except (KeyError, TypeError):
        return False


def extract_pure_id(pure_record: PureRecord) -> PureID:
    """Extract pure id."""
    return pure_record["pureId"]


def extract_file_url(pure_record: PureRecord) -> URL:
    """Extract file url."""

    def condition(item):
        return access_type(item) in ["Open", "Offen"] and license_type(item).startswith(
            "CC BY"
        )

    file_urls = []
    for electronic_version in pure_record["electronicVersions"]:
        try:
            file_url = electronic_version["file"]["fileURL"]
            if condition(electronic_version):
                file_urls.append(file_url)
        except (KeyError, TypeError):
            continue

    if len(file_urls) == 0:
        raise PureRuntimeError(pure_record)

    return file_urls
